<?php
/**
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations under the 
 * License.
 * 
 * The Original Code is Alchitect Core Server.
 * 
 * The Initial Developer of the Original Code is Darrell Brogdon.
 * Portions created by Darrell Brogdon are 
 * Copyright (C) 2002-2009 Alchisoft, LLC
 * All Rights Reserved.
 * 
 * Contributor(s): .
 * 
 * @package AlchitectCoreServer
 */
/**
 * An abstract class for implementing the Active Record pattern.
 * 
 * You must implement {@link initTable()} and {@link initDatabaseType()}. 
 * You are encouraged to also implement at least {@link initDatabase()} as
 * well as {@link initRelations()} if applicable.
 * 
 * Based on work by Leendert Brouwer, Matthijs Tempels.
 * 
 * @abstract 
 * @copyright (c)2005-2009 Leendert Brouwer, Matthijs Tempels, Darrell Brogdon
 * @package AlchitectCoreServer
 * @version $Id$
 */
abstract class ASActiveRecord extends ASApplication implements ASObserver
{
	/**
	 * A list of constraints against the current table.
	 * 
	 * Constraints are used in {@link update()} to build the 'AND' clauses.
	 *
	 * @access protected
	 * @see addConstraint()
	 * @see getPairedConstraintFields()
	 * @var Array
	 */
	protected $constraints = array();
	
	/**
	 * The name of the current database.
	 *
	 * @access protected
	 * @var String
	 */
	protected $database = '';
	
	/**
	 * The database type.
	 * 
	 * This is used to determine which loader to use.
	 *
	 * @access protected
	 * @see ASMySQLLoader.php
	 * @see ASPgSQLLoader.php
	 * @var String
	 */
	protected $database_type = 'MySQL';
	
	/**
	 * The database handle resource.
	 *
	 * @access protected
	 * @var Resource
	 */
	protected $dbh = null;
	
	/**
	 * Keeps track of whether a record exists.
	 *
	 * @access protected
	 * @var Boolean
	 */
	protected $exists = false;
	
	/**
	 * An array of fields.
	 * 
	 * When the database analysis is performed the table field information
	 * is stored in this array.
	 *
	 * @access protected
	 * @see loadFields()
	 * @var Array
	 */
	protected $field_objects = array();
	
	/**
	 * An array containing one-to-many mappings with foreign keys.
	 *
	 * @access protected
	 * @see getOneToMany()
	 * @see loadForeignsOneToMany()
	 * @var Array
	 */
	protected $foreign_one_to_manys_objects = array();
	
	/**
	 * @todo Add comment
	 *
	 * @access protected
	 * @var Array
	 */
	protected $foreign_one_to_one_objects = array();
	
	/**
	 * Whether foriegn tables have been loaded.
	 *
	 * @access protected
	 * @var Boolean
	 */
	protected $foreigns_loaded = false;
	
	/**
	 * An array of 'GROUP BY' fields.
	 *
	 * @access protected
	 * @var Array
	 */
	protected $group_by_fields = array();
	
	/**
	 * The name of the hook table.
	 *
	 * @access protected
	 * @var String
	 */
	protected $hook_table = '';
	
	/**
	 * An array of attribs to be ignored.
	 *
	 * @access protected
	 * @var Array
	 */
	protected $ignore_attribs = array();

	/**
	 * @todo Add Comment
	 */
	protected $last_insert_id = null;
	
	/**
	 * An array of Observable objects
	 *
	 * @access protected
	 * @var Array
	 */
	protected $obervables = array();
	
	/**
	 * @todo Add comment
	 *
	 * @access protected
	 * @var Array
	 */
	protected $one_to_manys = array();
	
	/**
	 * @todo Add comment
	 *
	 * @access protected
	 * @var String
	 */
	protected $one_to_one_method = '';
	
	/**
	 * @todo Add comment
	 *
	 * @access protected
	 * @var Array
	 */
	protected $one_to_ones_f2p = array();
	
	/**
	 * @todo Add comment
	 *
	 * @access protected
	 * @var Array
	 */
	protected $one_to_ones_p2f = array();
	
	/**
	 * @todo Add comment
	 *
	 * @access protected
	 * @var Array
	 */
	protected $orderby_fields = array();
	
	/**
	 * @todo Add Comment
	 */
	protected $row_count = 0;
	
	/**
	 * The name of the current table.
	 *
	 * @access protected
	 * @var String
	 */
	protected $table;

	/**
	 * Foreign Key to Primary Key constant.
	 *
	 */
	const AS_AR_FOREIGN_TO_PRIMARY = 0;
	
	/**
	 * Primary Key to Forieng Key constant.
	 *
	 */
	const AS_AR_PRIMARY_TO_FOREIGN = 1;
	
	/**
	 * Construct the object.
	 * 
	 * Initialize the table and database, load the table fields and build 
	 * a list of constraints.
	 *
	 */
	function __construct()
	{
		parent::__construct();

		$this->initTable();
		$this->initDatabaseType();
		$this->initDatabase();
		$this->loadFields();
		
		$this->constraints[] = $this->getPrimaryKeyField();
	}
	
	/**
	 * Add a new field to the constraints array.
	 * 
	 * The primary key is automatically added as a constraint in the Constructor.
	 *
	 * @example ../../docs/examples/ASActiveRecord_addConstraint.php
	 * @param ASActiveRecordField The field to be used as a constraint.
	 */
	public function addConstraint(ASActiveRecordField $constraint_field)
	{
		$this->constraints[] = $constraint_field;
	}

	/**
	 * Add a field to the group_by_fields array.
	 *
	 * @param ASActiveRecordField The field to be used in a 'GROUP BY' clause.
	 */
	public function addGroupByField(ASActiveRecordField $field)
	{
		$this->group_by_fields[] = $field;
	}
	
	/**
	 * addObserver()
	 *
	 * @param Observer The observer object.
	 */
	public function addObserver(ASObservable $observer)
	{
		$this->obervables[] = $observer;
	}
	
	/**
	 * Add a new field to the orderby_fields array.
	 *
	 * @param String The field to be used in an 'ORDER BY' clause.
	 */
	public function addOrderByField($field)
	{
		$this->orderby_fields[] = $field;
	}
	
	/**
	 * Execute a SQL 'DELETE' query against the current table.
	 *
	 * @return Integer The number of rows deleted.
	 */
	public function delete()
	{
		$sql = "DELETE FROM {$this->table}";

		if (count($this->field_objects) > 0)
		{
			$sql .= ' WHERE ' . implode(' AND ', $this->getPairedFields());
		}

		$res = $this->query($sql);

		return $res->rowCount(); 
	}
	
	/**
	 * Completely remove an attribute from the $field_objects array.
	 * 
	 * @param String The name of the attrib to delete.
	 */
	public function deleteAttrib($key)
	{
		if (isset($this->field_objects[$key]))
		{
			unset($this->field_objects[$key]);
		}
	}
	
	/**
	 * Check whether data has been found in a get*() request.
	 *
	 * @return Boolean
	 */
	public function exists()
	{
		return $this->exists;
	}
	
	/**
	 * Check whether a field exists.
	 *
	 * @param String The name of the field to check.
	 * @see getFields()
	 */
	function fieldExists($field_name)
	{
		foreach ($this->getFields() as $field)
		{
			if ($field == $field_name)
			{
				return true;
			}
		}
		
		return false;
	}
	
	/**
	 * Perform a SQL 'SELECT' query against the current table.
	 * If the result set is not empty then returns true.
	 *
	 * @param Boolean Whether to load foreign fields.
	 * @return Boolean
	 */
	public function get($load_foreigns = false)
	{
		if (false == is_null($this->table))
		{
			$this->exists = false;
			$sql		  = "SELECT * FROM {$this->table}";
	
			if (count($this->field_objects) > 0)
			{
				$where = implode(' AND ', $this->getPairedFields($this->getFields()));
				
				if ($where)
				{
					$sql .= " WHERE $where";
				}
			}
	
			$res = $this->query($sql);
	
			if ($res)
			{
				$all_data	  = $res->fetchAll();
				$this->exists = sizeof($all_data);
				
				if (sizeof($all_data))
				{
					$this->setInternalData($all_data[0]);
				}
	
				if ($load_foreigns)
				{
					$this->loadForeigns();
				}
			}
	
			return $this->exists;
		}
		else
		{
			ASLog::instance()->warn('Attempted to fetch data on an undefined table');
		}
		
		return false;
	}
	
	/**
	 * Get all data from the current table.
	 * Returns an array of ActiveRecord objects.
	 *
	 * @return Array
	 */
	public function getAll()
	{
		$sql = "SELECT * FROM {$this->table}";
		
		if (count($this->getPairedFields()) > 0)
		{
			$sql .= ' WHERE ' . implode(' AND ', $this->getPairedFields());
		}
			
		if (count($this->orderby_fields) > 0)
		{
			$sql .= ' ORDER BY ' . implode(',', $this->orderby_fields);
		}

		if (count($this->group_by_fields) > 0)
		{
			$sql .= ' GROUP BY ' . implode(',', $this->group_by_fields);
		}

		$res      = $this->query($sql);
		$objects  = array();
		$all_data = $res->fetchAll();

		foreach ($all_data as $data)
		{
			$classname = get_class($this);
			$obj	   = new $classname($this->table);

			$obj->setInternalData($data);
			$obj->exists = true;
			
			$objects[] = $obj;
		}
		
		return $objects;
	}
	
	/**
	 * Get all data from the current table where given fields have values.
	 * Returns an array of ActiveRecord objects.
	 *
	 * @return Array
	 */
	public function getAllLike()
	{
		$sql = "SELECT * FROM {$this->table}";

		if (count($this->field_objects) > 0)
		{
			$sql .= ' WHERE '
				 . implode
				   (
				   		' AND '
				      , $this->getPairedLikeFields($this->field_objects)
				   );
		}
			
		if (count($this->orderby_fields) > 0)
		{
			$sql .= ' ORDER BY ' . implode(',', $this->orderby_fields);
		}
		
		$res     = $this->query($sql);
		$objects = array();

		foreach ($res->fetchAll() as $data)
		{
			$classname = get_class($this);
			$obj	= new $classname();

			$obj->setInternalData($data);
			$obj->exists = true;
			
			$objects[] = $obj;
		}
		
		return $objects;
	}
	
	/**
	 * Get all data from the current table where given fields have given values.
	 * Returns an array of ActiveRecord objects.
	 *
	 * @param String The 'WHERE' clause.
	 * @return Array
	 */
	public function getAllWhere($where_clause)
	{
		$sql     = "SELECT * FROM {$this->table} WHERE {$where_clause}";
		$res     = $this->query($sql);
		$objects = array();

		foreach ($res->fetchAll() as $data)
		{
			$classname = get_class($this);
			$obj	   = new $classname($this->table);

			$obj->setInternalData($data);
			$obj->exists = true;
			
			$objects[] = $obj;
		}
		
		return $objects;
	}
	
	/**
	 * Get the value of data for a specified key.
	 * Optionally return said value with the HTML entities translated.
	 *
	 * @param String The name of the attrib.
	 * @param Boolean Pass the result through htmlentities() before returning.
	 * @return String
	 */
	public function getAttrib($key, $htmlentities = true)
	{
		$gettername = 'get' . ucfirst($key);
		
		if (method_exists($this, $gettername))
		{
			return $this->$gettername();
		}
		else
		{
			if (isset($this->field_objects[$key]))
			{
				if ($htmlentities)
				{
					return htmlentities($this->field_objects[$key]->value());
				}
				else
				{
					return $this->field_objects[$key]->value();
				}
			}
			else
			{
				return false;
			}
		}
	}
	
	/**
	 * Perform a custom SQL query against the database.
	 *
	 * @param String Raw SQL to be executed.
	 * @return Boolean
	 */
	public function getBySQL($sql)
	{
		$this->exists = false;
		$res          = $this->query($sql);
		
		if ($res->rowCount() > 0)
		{
			$this->exists = true;
			$this->setInternalData($res->fetchAll());
		}
		
		if (false == $this->lazy)
		{
			$this->loadForeigns();
		}
		
		return $this->exists;
	}
	
	/**
	 * Get all the fetched data.
	 * 
	 * Usually called after {@link get()} is called.
	 *
	 * @return Array
	 */
	public function getData()
	{
		$data = array();
		
		foreach($this->field_objects as $field)
		{
			$data[$field->name()] = $this->getAttrib($field->name());
		}
		
		return $data;
	}
	
	/**
	 * Get all the fields or column names for the current table.
	 *
	 * @return Array
	 */
	public function getFields()
	{
		$fields = array();

		foreach($this->field_objects as $field)
		{
			$fields[] = $field->name();
		}

		return $fields;
	}
	
	/**
	 * Get all field objects for a given key.
	 *
	 * @param String The name of the field.
	 * @return Array
	 */
	public function getFieldObject($key)
	{	
		return $this->field_objects[$key];
	}

	/**
	 * Get all field objects.
	 *
	 * @return Array
	 */
	public function getFieldObjects()
	{
		return $this->field_objects;
	}
	
	/**
	 * Get the hook table.
	 *
	 * @return String
	 */
	protected function getHookTable()
	{
		return $this->hook_table;
	}
		
	/**
	 * @todo Add comment
	 *
	 * @param String The name of the object in the one-to-many map.
	 * @return Array
	 */
	public function getOneToMany($object_name)
	{
		return $this->foreign_one_to_manys_objects[$object_name];
	}
	
	/**
	 * @todo Add comment
	 *
	 * @param String The name of the object in the one-to-one map.
	 * @return Array
	 */
	public function getOneToOne($object_name)
	{
		return $this->foreign_one_to_one_objects[$object_name];
	}
	
	/**
	 * @todo Add comment
	 *
	 * @return Array
	 */
	protected function getPairedConstraintFields()
	{
		$paired = array();
		
		foreach($this->constraints as $constraint)
		{
			// XXX - Verify that this doesn't introduce bugs!
			if ('' == $constraint->value())
			{
				continue;
			}
			
			$paired[] = "{$constraint->name()} = {$this->dbh->quote($constraint->value())}";
		}				
		
		return $paired;
	}
	
	/**
	 * @todo Add comment
	 *
	 * @return Array
	 */
	protected function getPairedFields()
	{	
		$paired = array();		
		
		foreach ($this->field_objects as $field)
		{
			if (strlen(trim($field->value())) > 0 && $field->isTouched())
			{
				$paired[$field->name()]
					= "{$field->name()} = "
					. $this->dbh->quote($field->value());
			}
		}
		
		return $paired;
	}
	
	/**
	 * @todo Add comment
	 *
	 * @return Array
	 */
	protected function getPairedLikeFields()
	{
		$paired = array();
		
		foreach ($this->field_objects as $field)
		{
			$paired[$field->name()] = "{$field->name()} LIKE {$this->dbh->quote($field->value())}";
		}
		
		return $paired;
	}

	/**
	 * Get the field of the current table defined as the primary key.
	 *
	 * @return String
	 */
	public function getPrimaryKeyField()
	{
		foreach($this->field_objects as $field)
		{
			if ($field->isPrimary())
			{
				return $field;
			}
		}

		return false;
	}
	
	/**
	 * Get all unignored fields as quoted.
	 *
	 * @return Array
	 */
	protected function getQuotedFields()
	{
		$quoted = array();
		
		foreach ($this->getUnignoredAttribs() as $field)
		{
			$this->validateField($field);
			$this->sanitizeField($field);
			$quoted[$field->name()] = $this->dbh->quote($field->value());
		}
		
		return $quoted;
	}
	
	/**
	 * Get a list of attribs that have been flagged to be ignored.
	 *
	 * @return Array
	 */
	public function getUnignoredAttribs()
	{
		if (sizeof($this->ignore_attribs))
		{
			$ret = array();

			foreach ($this->getFieldObjects() as $key=>$val)
			{
				if (in_array($key, $this->ignore_attribs))
				{
					continue;
				}

				$ret[$key] = $val;
			}
		}
		else
		{
			$ret = $this->getFieldObjects();
		}

		return $ret;
	}
	
	/**
	 * Get all unignored fields raw.
	 *
	 * @return Array
	 */
	protected function getUnquotedFields()
	{
		$quoted = array();
		
		foreach ($this->getUnignoredAttribs() as $field)
		{
			$quoted[$field->name()] = $field->value();
		}
		
		return $quoted;
	}
	
	/**
	 * Get whether the current database has a hook table.
	 *
	 * @return Boolean
	 */
	protected function hasHookTable()
	{
		if (isset($this->hook_table) && strlen(trim($this->hook_table)) > 0)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * @todo Add comment
	 *
	 * @param Object The object to add to the one-to-many stack.
	 * @param String The name of the hook table of the object.
	 */
	public function hasMany($obj, $hook_table = false)
	{
		if ($hook_table)
		{
			$obj->setHookTable($hook_table);
		}
		
		$this->one_to_manys[] = $obj;	
	}
	
	/**
	 * @todo Add comment
	 *
	 * @param Object The object to add to the one-to-one-stack.
	 * @param Integer Primary-to-foriegn or Foreign-to-primary.
	 */
	public function hasOne($obj, $method=self::AS_AR_PRIMARY_TO_FOREIGN)
	{
		if ($method == self::AS_AR_PRIMARY_TO_FOREIGN)
		{
			$this->one_to_ones_p2f[] = $obj;
		}
		else
		{
			$this->one_to_ones_f2p[] = $obj;
		}
	}

	/**
	 * Initialize the database.
	 *
	 */
	public function initDatabase()
	{
		$this->dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
	}
	
	/**
	 * Initialize the database type.
	 * 
	 * Supported databases are currently MySQL and PostgreSQL.
	 * 
	 * The default is MySQL.
	 *
	 */
	abstract function initDatabaseType();

	/**
	 * Base method for initializing relations between tables.
	 *
	 */
	public function initRelations()
	{
	}
	
	/**
	 * Abstract method for initializing the table.
	 * 
	 * @example ASActiveRecord_initTable.php
	 *
	 */
	abstract function initTable();
	
	/**
	 * Execute a SQL 'INSERT' query against the current table.
	 * 
	 * Returns the ID of the inserted row.
	 *
	 * @example ASActiveRecord_insert.php
	 * @return Integer
	 */
	public function insert()
	{		
		$keys = array_keys($this->getUnignoredAttribs());
		$vals = $this->getQuotedFields();
		
		$sql = sprintf(
			"INSERT INTO %s (\n\t%s\n) VALUES (\n\t%s\n)"
		  , $this->table
		  , implode(', ', $keys)
		  , implode(', ', $vals)
		);

		try
		{
			$this->query($sql);
		
			$this->last_insert_id = $this->dbh->lastInsertId('id');
			
			return true;
		}
		catch (Exception $ex)
		{
			throw new Exception($ex->getMessage(), $ex->getCode());
		}
		
		return false;
	}

	public function lastInsertID()
	{
		return $this->last_insert_id;
	}
	
	/**
	 * Load all the fields of the current table.
	 */
	private function loadFields()
	{
		if ('' != $this->database_type)
		{
			if ('mysql' == strtolower($this->database_type))
			{
				$db_type = 'MySQL';
			}
			
			if ('pgsql' == strtolower($this->database_type))
			{
				$db_type = 'PgSQL';
			}
			
			$cls_name = "AS{$db_type}Loader";
			$loader   = new $cls_name();
			$fields   = $loader->loadFields($this->database, $this->table, $this->dbh);
							
			foreach ($fields as $field)
			{
	//			ASLog::instance()->debug("\tLoading field '{$field->name()}'");
				$this->field_objects[$field->name()] = $field;
			}
		}
	}
	
	/**
	 * Load all foreign key relationships for the current table.
	 */
	public function loadForeigns()
	{
		if (!$this->foreigns_loaded)
		{
			$this->initRelations();
			
			// load one to ones
			$this->loadForeignsOneToOne();
				
			// load one to many
			$this->loadForeignsOneToMany();
			
			$this->foreigns_loaded = true;
		}
	}
	
	/**
	 * Builds an array containing one-to-many mappings with foreign keys.
	 */
	protected function loadForeignsOneToMany()
	{
		foreach($this->one_to_manys as $relation)
		{
			$this->foreign_one_to_manys_objects[get_class($relation)] = array();
			
			if (!$relation->hasHookTable())
			{								
				$relation->setAttrib(
					$this->getPrimaryKeyField()->name(), 
					$this->getAttrib($this->getPrimaryKeyField()->name())					
				);
				
				foreach($relation->getAll() as $obj)
				{
					$this->foreign_one_to_manys_objects[get_class($relation)][]
						= $obj;
				}
			}
			else
			{		
				// query hooktable
				$sql = "
SELECT *
  FROM {$relation->getHookTable()}
 WHERE {$this->getPrimaryKeyField()->name()} = '{$this->getPrimaryKeyField()->value()}'";

				$res = $this->query($sql);
					
				foreach ($res->fetchAll() as $row)
				{
					$objName = get_class($relation);
					$obj	 = new $objName();
					
					$obj->setAttrib($this->getPrimaryKeyField()->name(), $row[$this->getPrimaryKeyField()->name()]);
					$obj->get();
					$this->foreign_one_to_manys_objects[get_class($relation)][] = $obj;
				}		
			}
		}
	}
	
	/**
	 * @todo Add comment
	 *
	 */
	protected function loadForeignsOneToOne()
	{
		//Primary to Foreign
		foreach ($this->one_to_ones_p2f as $relation)
		{			
			if ($this->getPrimaryKeyField())
			{				
				$relation->setAttrib(
					$this->getPrimaryKeyField()->name()
				  , $this->getAttrib($this->getPrimaryKeyField()->name())									
				);
				
				$relation->get();				
				$this->foreign_one_to_one_objects[get_class($relation)] = $relation;
				
			}			
		}
		
		//Foreign to Primary
		foreach ($this->one_to_ones_f2p as $relation)
		{
			if ($relation->getPrimaryKeyField())
			{				
				$relation->setAttrib(
					$relation->getPrimaryKeyField()->name()
				  , $this->getAttrib($relation->getPrimaryKeyField()->name())									
				);
				
				$relation->get();				
				$this->foreign_one_to_one_objects[get_class($relation)] = $relation;
			}			
		}
	}

	/**
	 * notifyObservables()
	 *
	 * @param info
	 */
	public function notifyObservables($info)
	{
		foreach ($this->obervables as $observable)
		{
			$observable->updateFromObserver($info);
		}
	}
	
	/**
	 * @todo Add comment
	 */
	private function query($sql)
	{
		if (is_null($this->dbh))
		{
			throw new ASNoDbHandleAvailableException();
		}
		
		ASLog::instance()->debug("Executing SQL query: \n$sql");
		
		$res = $this->dbh->query($sql, PDO::FETCH_ASSOC);
		
		if (($this->dbh->errorCode()
		&&   $this->dbh->errorCode() != '00000')
		||  false === $res)
		{
			$err_info = $this->dbh->errorInfo();
			
			throw new ASException($err_info[2], (integer)$err_info[0]);
		}
		
		return $res;
	}
	
	/**
	 * Quote a given value.
	 *
	 * @param String Escape quotes within a value to ensure it can be properly
	 * inserted or updated in the database.
	 * @return String
	 */
	private function quote($value)
	{
		if (get_magic_quotes_gpc())
		{
			$value = stripslashes($value);
		}								
		
		$value = $this->dbh->quote($value);
				
		return $value;
	}
	
	/**
	 * removeObserver()
	 *
	 * @param observable
	 */
	public function removeObserver($observable)
	{
		unset($this->obervables[$observable]);
	}
	
	/**
	 * Sanitize the value of a field before it is inserted or updated in
	 * the database.
	 *
	 * @param ASActiveRecordField The field containing the value to be
	 * sanitized.
	 * @return Void
	 */
	protected function sanitizeField(ASActiveRecordField $field)
	{
		$rule = $field->sanitizationRule();
			
		if ('' != $rule)
		{			
			$filter_str	= "FILTER_SANITIZE_" . strtoupper($rule);
			$result		= filter_var($field->value(), constant($filter_str));
			
			$field->setValue($result);
		}
	}
	
	/**
	 * Set the value for a given attribute.
	 *
	 * @param String The name of the attribute.
	 * @param Mixed The value of the attribute.
	 * @return Boolean
	 */
	public function setAttrib($key, $val)
	{
		$settername = 'set'.ucfirst($key);
		
		if (method_exists($this, $settername))
		{
			$this->field_objects[$key]->setValue($this->$settername($val));
		}
		else
		{
			if (isset($this->field_objects[$key]) && is_object($this->field_objects[$key]))
			{
				$this->field_objects[$key]->setValue($val);
			}
		}
		
		return $this;
	}
	
	/**
	 * Set attributes from a form POST.
	 */
	public function setAttribsFromPost()
	{
		if (isset($_POST))
		{
			foreach ($_POST as $key => $val)
			{
				if (in_array($key, $this->getFields()))
				{
					$this->setAttrib($key, $val);
				}
			}
		}
	}
	
	/**
	 * Set attributes from an HTTP request (GET or POST most likely).
	 */
	public function setAttribsFromRequest()
	{
		if (isset($_REQUEST))
		{
			foreach ($_REQUEST as $key => $val)
			{
				if (in_array($key, $this->getFields()))
				{
					$this->setAttrib($key, $val);
				}
			}
		}
	}
	
	/**
	 * Set the attributes from a given data array.
	 *
	 * @param Array An array of data to be set.
	 */
	public function setData($data)
	{
		foreach ($data as $key => $val)
		{
			$this->setAttrib($key, $val);
		}		
	}
	
	/**
	 * Set the name of the hook table.
	 *
	 * @param String The name of the hook table.
	 */
	protected function setHookTable($table)
	{
		$this->hook_table = $table;
	}
	
	/**
	 * Add to the list of ignored attributes.
	 *
	 * @param String The name of the attribute.
	 */
	public function setIgnoredAttrib($attrib)
	{
		$this->ignore_attribs[] = $attrib;
	}
	
	/**
	 * @todo Add comment
	 *
	 * @param Array $data
	 */
	protected function setInternalData($data)
	{
		if (is_array($data) && sizeof($data))
		{
			foreach ($data as $key => $val)
			{
				if (is_numeric($key)) continue;
				$this->field_objects[$key]->setInternalValue($val);			
			}				
		}
	}
	
	/**
	 * Unset a given attribute.
	 *
	 * @param String The name of the attribute.
	 */
	public function unsetAttrib($key)
	{
		if (isset($this->field_objects[$key])
		 && strlen(trim($this->field_objects[$key]->value())) > 0)
		{
			$this->field_objects[$key]->setValue('');
		}
	}
	
	/**
	 * Execute a SQL 'UPDATE' query against the current table.
	 *
	 * @return Integer
	 */
	public function update()
	{
		$sql = "
UPDATE {$this->table}
   SET " . implode(",\n       ", $this->getPairedFields($this->getFields())) . "
";
		
		if (count($this->field_objects) > 0)
		{
			$sql .= " WHERE "
				 . implode(' AND ', $this->getPairedConstraintFields());
		}
		
		$res				= $this->query($sql);		
		$this->row_count	= $res->rowCount();

		if ($res)
		{
			return true;
		}
		
		return false;
	}

	/**
	 * Perform a validation on a field before it is inserted or updated in
	 * the database.
	 *
	 * @param ASActiveRecordField The field on which to perform the validation.
	 * @return Void
	 */
	protected function validateField(ASActiveRecordField $field)
	{
		$result	= false;
		$rule	= $field->validationRule();
			
		if ('' != $rule)
		{			
			$filter_str	= "FILTER_VALIDATE_" . strtoupper($rule);
			$result		= filter_var($field->value(), constant($filter_str));
			
			if (false === $result)
			{
				// XXX - This should be auto-included in __autoload()
				include
				(	
					ALCHITECT_CORE_SERVER_EXCEPTIONS_ROOT
				  . '/ASValidationException.php'
				);
				throw new ASValidationException($field->value(), $rule);
			}
			
			$field->setValue($result);
		}
	}
}
?>
