<?php
/**
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is Alchitect Core Server.
 * 
 * The Initial Developer of the Original Code is Darrell Brogdon.
 * Portions created by Darrell Brogdon are Copyright (C) 2002-2009 Alchisoft, LLC
 * All Rights Reserved.
 * 
 * Contributor(s): .
 * 
 * @package AlchitectCoreServer
 */
/**
 * Class for initializing the database.  Typically the parent class to the
 * Model classes.
 *
 * @author Darrell Brogdon <darrell@brogdon.net>
 * @copyright (c)2002-2009 Alchisoft, LLC
 * @package AlchitectCoreServer
 * @version $Id$
 */
class ASDatabase extends ASActiveRecord
{
	/**
	 * The database handle.
	 * 
	 * The connection handle returned from PDO.
	 *
	 * @access protected
	 * @var Resource
	 */
	protected $dbh = null;

	/**
	 * The constructor.
	 *
	 * @access public
	 * @param String The name of the table.
	 * @param Boolean Whether to initialize the database connection.
	 */
	function __construct($table=false, $init=true)
	{	
		if ('' != ASConfig::instance()->databaseType())
		{
			if ($init && is_null($this->dbh))
			{
				try
				{
					$this->init();
	
					if ($table)
					{
						$this->setTable($table);
				
						parent::__construct();
					}
				}
				catch (Exception $ex)
				{
//					ASLog::instance()->warn("database may be down: " . $ex->getMessage());
				}
			}
		}
	}

	/**
	 * Generic accessor method.
	 * 
	 * Access the value of a property using $name as the key.
	 *
	 * @access public
	 * @param String The name of the property to get.
	 * @return Mixed
	 */
	public function __get($name)
	{
		if (isset($this->$name))
		{
			return $this->$name;
		}
		
		return null;
	}

	/**
	 * Get the connection handle.
	 *
	 * @access public
	 * @return Resource
	 */
	public function handle()
	{
		return $this->dbh;
	}
	
	/**
	 * Create a new database.
	 *
	 * @access public
	 * @param String $db_name The name of the database
	 * @return Void
	 */
	public function createDb($db_name)
	{
		$this->dbh->exec("CREATE DATABASE $db_name");
	}
	
	/**
	 * Initialize the database type.
	 * 
	 * The actual type is defined in the config file.
	 *
	 * @access public
	 * @return Void
	 */
	function initDatabaseType()
	{
		$this->database_type = ASConfig::instance()->databaseType();
	}

	/**
	 * Initialize the database connection.
	 * 
	 * Initializes a PDO database connection and stores it in the local property
	 * $dbh. Currently only MySQL and PostgreSQL are supported.  All of the 
	 * params are optional and if left blank will be obtained from the config 
	 * file.  This is the preferred use.
	 *
	 * @access public
	 * @param String The database type (i.e. mysql, postgresql, etc...)
	 * @param String The hostname of the database server
	 * @param String The name of the database
	 * @param String The username who has access to the database
	 * @param String The password associated with $db_user
	 * @return Void
	 */
	public function init($db_type=false, $db_host=false, $db_name=false, $db_user=false, $db_pass=false)
	{		
		$db_type = ($db_type ? $db_type : trim(strtolower(ASConfig::instance()->databaseType())));
		$db_host = ($db_host ? $db_host : trim(strtolower(ASConfig::instance()->databaseHost())));
		$db_name = ($db_name ? $db_name : trim(strtolower(ASConfig::instance()->databaseName())));
		$db_user = ($db_user ? $db_user : trim((string)ASConfig::instance()->databaseUser()));
		$db_pass = ($db_pass ? $db_pass : trim((string)ASConfig::instance()->databasePassword()));
		
		try
		{
			$pdo_connection_format	= "%s:host=%s;dbname=%s";
			$pdo_connection_str		= sprintf($pdo_connection_format, $db_type, $db_host, $db_name);
			
			$this->dbh = new PDO($pdo_connection_str, $db_user, $db_pass);
		}
		catch (Exception $ex)
		{
			trigger_error('ERROR: ' . $ex->getCode() . ': ' . $ex->getMessage());
		}
	}	
	
	/**
	 * Initialize the database name.
	 * 
	 * This value is defined in the config file.
	 *
	 * @access public
	 * @return Void
	 */
	public function initDatabase()
	{
		$this->database = ASConfig::instance()->databaseName();
	}

	/**
	 * Initialize the database table.
	 *
	 * @access public
	 * @see table()
	 * @return Void
	 */
	public function initTable()
	{
		$this->table = $this->table();
	}

	/**
	 * Set the table name.
	 *
	 * @access public
	 * @param String
	 * @return Void
	 */
	public function setTable($table)
	{
		$this->table = $table;
	}

	/**
	 * Get the table name.
	 *
	 * @access public
	 * @return String
	 */
	public function table()
	{
		return $this->table;
	}
}
?>
