<?php
/**
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is Alchitect Core Server.
 * 
 * The Initial Developer of the Original Code is Darrell Brogdon.
 * Portions created by Darrell Brogdon are Copyright (C) 2002-2009 Alchisoft, LLC
 * All Rights Reserved.
 * 
 * Contributor(s): .
 * 
 * @package AlchitectCoreServer
 */
/**
 * Define global log level constants.
 * 
 * This is done here rather than making them class constants to make it easier to use.
 */
define('AS_LOG_LEVEL_INFO',  1);
define('AS_LOG_LEVEL_DEBUG', 2);
define('AS_LOG_LEVEL_WARN',  4);
define('AS_LOG_LEVEL_FATAL', 8);

/**
 * ASLog
 * 
 * A simple logging class.  Log data by default is stored in 
 * /var/log/alchitect.log.
 * 
 * This class is a Singleton and should always be accessed by calling
 * 
 * <pre>ASLog::instance()</pre>
 * 
 * @author Darrell Brogdon <darrell@brogdon.net>
 * @copyright (c)2002-2009 Alchisoft, LLC
 * @package AlchitectCoreServer
 * @version $Id$
 */
class ASLog
{
	/**
	 * The instance of this object.
	 * 
	 * @static 
	 * @access private
	 * @var ASLog
	 */
	static private $instance = false;
	
	/**
	 * The name of the file to which to log.
	 * 
	 * @access private
	 * @var String
	 */
	private $log_file = null;
	
	/**
	 * The current log level.
	 * 
	 * @access private
	 * @var Integer
	 */
	private $log_level = 0;
	
	/**
	 * The log prefix.  Prepended to every log entry.
	 * 
	 * @access private
	 * @var $log_prefix
	 */
	private $log_prefix = 'AS';
	
	/**
	 * The log output type.  Currently only 'file' is supported.
	 * 
	 * @access private
	 * @var String
	 */
	private $log_type = 'file';
	
	/**
	 * The log message format.
	 * 
	 * The default format is: <pre>M d H:i:s PREFIX[LEVEL]: MESSAGE</pre>
	 * 
	 * @access private
	 * @var String
	 */
	private $message_format = '%date %prefix[%level]: %message';
	
	/**
	 * The same as $message format but includes the current file and line where 
	 * the log method is called.
	 * 
	 * The default format is:
	 * <pre>M d H:i:s PREFIX[LEVEL] (__FILE__,__LINE__): MESSAGE</pre>
	 * 
	 * @access private
	 * @see $message_format
	 * @var String
	 */
	private $message_format_with_file_and_line 
		= '%date %prefix[%level] (%file,%line): %message';
	
	/**
	 * The date format.
	 * 
	 * Used in $message_format and $message_format_with_file_and_line
	 * 
	 * @access private
	 * @see $message_format
	 * @see $message_format_with_file_and_line
	 * @var String
	 */
	private $date_format = 'M d H:i:s';
	
	/**
	 * An array of supported log levels.
	 * 
	 * The levels are:
	 * <ul>
	 * 	<li>Info - <pre>AS_LOG_LEVEL_INFO</pre></li>
	 * 	<li>Debug - <pre>AS_LOG_LEVEL_DEBUG</pre></li>
	 * 	<li>Warn - <pre>AS_LOG_LEVEL_WARN</pre></li>
	 * 	<li>Fatal - <pre>AS_LOG_LEVEL_FATAL</pre></li>
	 * </ul>
	 * 
	 * @access private
	 * @var Array
	 */
	private $log_levels = array();
	
	/**
	 * The constructor.
	 * 
	 * Since this class is a Singleton this method cannot be called directly.
	 *
	 * @access private
	 * @param Integer
	 * @return Void
	 */
	private function __construct($level=false)
	{
		$this->log_levels = array(
			AS_LOG_LEVEL_INFO  => 'info'
		  , AS_LOG_LEVEL_DEBUG => 'debug'
		  , AS_LOG_LEVEL_WARN  => 'warn'
		  , AS_LOG_LEVEL_FATAL => 'fatal'
		);
		
		$this->log_level     = ($level ? $level : ASConfig::instance()->logLevel());
		$this->log_type      = ASConfig::instance()->logType();
		$this->log_file      = ALCHITECT_BASE_ROOT . '/' . ASConfig::instance()->logFilePath();
		
		if (false == is_dir(ALCHITECT_BASE_ROOT . '/logs'))
		{
			mkdir(ALCHITECT_BASE_ROOT . '/logs');
		}
		
		if (false == is_file($this->log_file))
		{
			touch($this->log_file);
		}
		
		if (false == is_writeable($this->log_file))
		{
			throw new Exception("Cannot write to log file '{$this->log_file}'");
		}
	}
	
	/**
	 * Convenience method for logging DEBUG information.
	 * 
	 * @access public
	 * @param String The message to log.
	 * @param String The file on which this method was called.
	 * @param Integer The line in the file on which this method was called.
	 * @return Void
	 */
	public function debug($message, $file=null, $line=null)
	{
		$this->log(AS_LOG_LEVEL_DEBUG, $message, $file, $line);
	}
	
	/**
	 * Convenience method for logging FATAL information.
	 * 
	 * @access public
	 * @param String The message to log.
	 * @param String The file on which this method was called.
	 * @param Integer The line in the file on which this method was called.
	 * @return Void
	 */
	public function fatal($message, $file=null, $line=null)
	{
		$this->log(AS_LOG_LEVEL_FATAL, $message, $file, $line);
	}
	
	/**
	 * Convenience method for logging INFO information.
	 * 
	 * @access public
	 * @param String The message to log.
	 * @param String The file on which this method was called.
	 * @param Integer The line in the file on which this method was called.
	 * @return Void
	 */
	public function info($message, $file=null, $line=null)
	{
		$this->log(AS_LOG_LEVEL_INFO, $message, $file, $line);
	}
	
	/**
	 * Get an instance of the ASLog object.
	 *
	 * @access public
	 * @param Integer
	 * @return Object
	 */
	static function instance($level=false)
	{
		if (false == self::$instance)
		{
			self::$instance = new ASLog($level);
		}
		
		return self::$instance;
	}
	
	/**
	 * Send a log message to the log file.
	 *
	 * @access public
	 * @param String The log level.
	 * @param String The message to log.
	 * @param String The file on which this method was called.
	 * @param Integer The line in the file on which this method was called.
	 * @return Void
	 */
	public function log($level, $message, $file=null, $line=null)
	{
		if (true == ASConfig::instance()->logEnabled() && $level <= $this->log_level)
		{
			if (is_array($message) || is_object($message))
			{
				$message = print_r($message, true);
			}

			file_put_contents
			(
				$this->log_file
			  , $this->parseMessage($level, $message, $file, $line) . "\n"
			  , FILE_APPEND
			);
		}
	}
	
	/**
	 * Get the current log level
	 * 
	 * @access public
	 * @return Integer
	 */
	public function logLevel()
	{
		return $this->log_level;
	}
	
	/**
	 * Parse a message based on $message_format
	 *
	 * @access private
	 * @param Integer The log level
	 * @param String The raw log message.
	 * @param String The PHP __FILE__ magic var.
	 * @param Integer The PHP __LINE__ magic var.
	 * @return String
	 */
	private function parseMessage($level, $message, $file=null, $line=null)
	{
		if (false == is_null($file) || false == is_null($line))
		{
			$formatted_msg = str_replace
			(
				'%prefix'
			  , $this->log_prefix
			  , $this->message_format_with_file_and_line
			);
		}
		else
		{
			$formatted_msg = str_replace
			(
				'%prefix'
			  , $this->log_prefix
			  , $this->message_format
			);
		}

		$formatted_msg = str_replace
		(
			'%date'
		  , date($this->date_format)
		  , $formatted_msg
		);
		$formatted_msg = str_replace
		(
			'%level'
		  , $this->log_levels[$level]
		  , $formatted_msg
		);
		$formatted_msg = str_replace('%file', $file, $formatted_msg);
		$formatted_msg = str_replace('%line', $line, $formatted_msg);
		$formatted_msg = str_replace('%message', $message, $formatted_msg);
		
		return $formatted_msg;
	}
	
	/**
	 * Override the log level defined in the config file.
	 * 
	 * @access public
	 * @param Integer The log level
	 * @return Void
	 */
	public function setLogLevel($log_level)
	{
		$this->log_level = $log_level;
	}
	
	/**
	 * Convenience method for logging WARN information.
	 * 
	 * @access public
	 * @param String $message The message to log.
	 * @param String $file The file on which this method was called.
	 * @param Integer $line The line in the file on which this method was called.
	 * @return Void
	 */
	public function warn($message, $file=null, $line=null)
	{
		$this->log(AS_LOG_LEVEL_WARN, $message, $file, $line);
	}
}
?>