<?php
/**
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is Alchitect Core Server.
 * 
 * The Initial Developer of the Original Code is Darrell Brogdon.
 * Portions created by Darrell Brogdon are Copyright (C) 2002-2009 Alchisoft, LLC
 * All Rights Reserved.
 * 
 * Contributor(s): .
 * 
 * @package AlchitectCoreServer
 */
/**
 * Class for managing page requests.
 *
 * @author Darrell Brogdon <darrell@brogdon.net>
 * @copyright (c)2002-2009 Alchisoft, LLC
 * @package AlchitectCoreServer
 * @version $Id$
 */
class ASPageRequest implements ASObserver
{
	/**
	 * A constant used by the observer indicating when the page has been 
	 * loaded
	 * 
	 * @var Constant 
	 */
	const AS_PAGE_REQ_NOTICE_PAGE_LOADED = 1000;
	
	/**
	 * A constant used by the observer indicating when the page has been
	 * requested through GET.
	 * 
	 * @var Constant
	 */
	const AS_PAGE_REQ_NOTICE_GET_REQUESTED = 1010;
	
	/**
	 * A constant used by the observer indicating when the page has been
	 * requested through POST.
	 * 
	 * @var Constant
	 */
	const AS_PAGE_REQ_NOTICE_POST_REQUESTED = 1020;
	
	/**
	 * A constant representing the part of the REST URL that represents
	 * the the index.  For Alchitect this should always be 'app'.
	 * 
	 * @var Constant
	 */
	const AS_PAGE_REQ_URL_PART_INDEX = 0;
	
	/**
	 * A constant representing the part of the REST URL that represents
	 * the page.
	 * 
	 * For example; given the URL 'http://example.com/app/users/list' the 
	 * part that AS_PAGE_REQ_URL_PART_PAGE represents is 'users'.
	 * 
	 * @var Constant.
	 */
	const AS_PAGE_REQ_URL_PART_PAGE = 1;
	
	/**
	 * A constant representing the part of the REST URL that represents
	 * the action.
	 * 
	 * For example; given the URL 'http://example.com/app/users/list' the 
	 * part that AS_PAGE_REQ_URL_PART_ACTION represents is 'list'.
	 * 
	 * @var Constant.
	 */
	const AS_PAGE_REQ_URL_PART_ACTION = 2;
	
	/**
	 * A constant representing the part of the REST URL that represents
	 * the id.
	 * 
	 * For example; given the URL 'http://example.com/app/users/edit/1' the 
	 * part that AS_PAGE_REQ_URL_PART_ID represents is '1'.
	 * 
	 * @var Constant.
	 */
	const AS_PAGE_REQ_URL_PART_ID = 3;
	
	/**
	 * A constant representing the part of the REST URL that represents
	 * the arguments.
	 * 
	 * For example; given the URL 'http://example.com/app/user/99/foo/bar' the 
	 * part that AS_PAGE_REQ_URL_PART_ARGS represents is everything after '99'.
	 * 
	 * @var Constant.
	 */
	const AS_PAGE_REQ_URL_PART_ARGS = -1;
	
	/**
	 * The page action.
	 *
	 * @access private
	 * @var String
	 */
	private $action = null;
	
	/**
	 * Anything in the URL after the id.
	 * 
	 * @access private
	 * @var Array
	 */
	private $args = array();
	
	/**
	 * The app name.
	 *
	 * @access private
	 * @var String
	 */
	private $app_name = '';
	
	/**
	 * The confirmation to be presented to the user that the requested action 
	 * performed successfully.
	 *
	 * @access private
	 * @var String
	 */
	private $confirmation = '';
	
	/**
	 *
	 */
	private $contents = '';
	
	/**
	 * The error to be presented to the user that the requested action failed.
	 *
	 * @access private
	 * @var String
	 */
	private $error = '';
	
	/**
	 * The URL to be used in the 'action' attribute of form elements.
	 * 
	 * @access private
	 * @var String
	 */
	private $form_action_url = '';
	
	/**
	 * The ID part of the URL.
	 * 
	 * @access private
	 * @var Mixed 
	 */
	private $id = '';
	
	/**
	 * The instance of this object.
	 * 
	 * @static 
	 * @access private
	 * @var $instance
	 */
	static private $instance = null;
	
	/**
	 * An array of objects observing this object.
	 *
	 * @access private
	 * @var Array
	 */
	private $observers = array();
	
	/**
	 * The name of the class associated with this page.
	 * 
	 * @access private
	 * @var String
	 */
	private $page_class_name = '';
	
	/**
	 * The data to be used on the page.
	 */
	private $page_data = null;
	
	/**
	 * The name of the page.
	 *
	 * @access private
	 * @var String
	 */
	private $page_name = '';
	
	/**
	 * The name of the template file.
	 * 
	 * @access private
	 * @var String
	 */
	private $template = '';
	
	/**
	 * A list of REST-based URL data.
	 *
	 * @access private
	 * @var Array
	 */
	private $url_parts = array(0 => 'app');

	/**
	 * The constructor.
	 * 
	 * Initializes the Model and Controller for the given page name.  If no
	 * page name is given then the REST-based URL is parsed.
	 * 
	 * Also notifies any observers.
	 *
	 * @access public
	 * @param String The page name.
	 * @return void
	 */
	private function __construct($page_name=null)
	{
		if (false == is_null($page_name))
		{
			$this->setPageName($page_name);
			$this->setPageClassName($this->pageName());
			$this->setAppName($this->pageName());
		}
		else
		{
			$this->parseURL();
		}

		$this->setModelName($this->pageClassName());
		$this->initController($this->pageClassName());
		$this->notifyObservables
		(
			array
			(
				'code' => self::AS_PAGE_REQ_NOTICE_PAGE_LOADED
			  , 'data' => array
			  	(
			  		'page_name' => $this->pageName()
				  , 'url_parts'  => $this->URLParts()
			    )
			)
		);

		if ('POST' == $_SERVER['REQUEST_METHOD'])
		{
			$this->notifyObservables
			(
				array
				(
					'code' => self::AS_PAGE_REQ_NOTICE_POST_REQUESTED
				  , 'data' => $_POST
				)
			);
		}

		$this->notifyObservables
		(
			array
			(
				'code' => self::AS_PAGE_REQ_NOTICE_GET_REQUESTED
			  , 'data' => $_GET
			)
		);
	}

	/**
	 * The action to perform for this page.
	 * 
	 * @access public
	 * @return String
	 */
	public function action()
	{
		return $this->action;
	}

	/**
	 * Get all the parts of the URL after the ID.
	 * 
	 * @access public
	 * @return Array
	 */
	public function args()
	{
		if (false == sizeof($this->args))
		{
			foreach ($this->url_parts as $idx=>$part) {
				if ($idx > self::AS_PAGE_REQ_URL_PART_ID)
				{
					$this->args[] = $part;
				}
			}
		}
		
		return $this->args;
	}
	
	/**
	 * Add an Observer to the observers stack
	 * 
	 * @access public
	 * @param ASObservable The observer
	 * @return void
	 */
    public function addObserver(ASObservable $observer)
    {
		$this->observers[] =& $observer;
	}

	/**
	 * Get the app name for this page.
	 * 
	 * If the URL looks like 'http://example.com/app/first_names/list' then the 
	 * app name is 'first_names'.  
	 * 
	 * @access public
	 * @return String
	 */
	public function appName()
	{
		return $this->app_name;
	}

	/**
	 * Get the confirmation for this page.
	 * 
	 * The confirmation is usually set after an action (i.e. form submit) is 
	 * performed.
	 * 
	 * @access public
	 * @return String
	 */
	public function confirmation()
	{
		return $this->confirmation;
	}
	
	/**
	 *
	 */
	public function displayPage()
	{
		echo $this->contents;
	}

	/**
	 * Get the error for this page.
	 * 
	 * The error is usually set after an action (i.e. form submit) is performed.
	 * 
	 * @access public
	 * @return String
	 */
	public function error()
	{
		return $this->error;
	}

	/**
	 * Get the URL for the action attribute of an HTML Form.
	 * 
	 * The format for the form action URL is: 
	 * $_SERVER['SCRIPT_NAME']/$page_name/$action/$urlData[4].
	 * 
	 * @access public
	 * @example ../../docs/examples/ASPageRequest_formActionURL.php
	 * @return String 
	 */
	public function formActionURL()
	{
		return $this->form_action_url;
	}
	
	/**
	 *
	 */
	public function handleAction()
	{
		$include_file = ALCHITECT_BASE_ROOT . "/pages/{$this->pageName()}/{$this->action()}.php";
		
		if (is_file($include_file))
		{
			include($include_file);
		}
		else
		{
			$this->setError('ERROR: The requested action cannot be handled.');
		}
	}

	/**
	 * Get the page to be included.
	 * 
	 * This is for the template page.  It determines which page to include and 
	 * if that file exists.  So given the URL 
	 * 'http://example.com/app/first_names/list/1234' the include file should be 
	 * found in 'pages/first_names/list.php' which is what this method will 
	 * return.
	 * 
	 * @access public
	 * @return String
	 */
	public function includeFileName()
	{
		$include_page = ('' == $this->action() ? $this->pageName() : $this->pageName() . '/' . $this->action());
		$include_page = "pages/{$include_page}.php";
		
		if (is_file($include_page))
		{
			ASLog::instance()->debug("Including page '{$include_page}'");
			return $include_page;
		}
		else
		{
			throw new Exception("File '{$include_page}' does not exist");
		}
	}

	/**
	 * The ID of this page.
	 * 
	 * Given the URL 'http://example.com/app/first_names/list/1234' the ID would be '1234'.
	 * 
	 * @access public
	 * @return Integer
	 */
	public function id()
	{
		return $this->id;
	}

	/**
	 * Include the page requested.
	 * 
	 * If the $page param is 'default' then 'pages/[PAGE NAME]/[PAGE NAME].php'
	 * is included.
	 * 
	 * If the file can't be found then 'pages/file_not_found.php' is included.
	 * 
	 * @example ../../docs/examples/ASPageRequest_includePage.php
	 * @param String The name of the page to included.
	 * @return Object Instance of ASPageRequest
	 */
	public function includePage($page, $enable_buffering=false)
	{
		global $Controller;
		
		$include_page = 'pages/file_not_found.php';
		
		if (true === $enable_buffering)
		{
			ob_start();
		}
		
		if ('default' == $page)
		{
			$inc_page = 'pages/'
					  . $this->pageName()
					  . '/'
					  . 'default.php';
			
			if (file_exists($inc_page))
			{
				$include_page = $inc_page;
			}
			else if (file_exists('pages/' . $this->pageName() . '.php'))
			{
				$include_page = 'pages/' . $this->pageName() . '.php';
			}
		}
		else
		{
			if (file_exists('pages/' . $page . '.php'))
			{
				$include_page = 'pages/' . $page . '.php';
			}
			else
			{
				return $this;
			}
		}
		
		include($include_page);
		
		if (true === $enable_buffering)
		{
			$this->contents .= ob_get_contents();
		
			ob_end_clean();
		}
		
		return $this;
	}
	
	/**
	 * Initialize the controller for this page.
	 * 
	 * Looks in the 'controllers/' directory for a file named 
	 * '{$page_name}Controller.php'.
	 * 
	 * If the file is found it is included and initialized and stored in 
	 * $GLOBALS['Controller'].  This allows any page to access its controller 
	 * through $Controller->method().
	 * 
	 * @access public
	 * @return void
	 */
	public function initController($class_name)
	{
		$ctrl_cls_name  		= ASString::camelize($class_name) . 'Controller';
		$classes_dir    		= 'controllers';
		$pages_dir				= ASString::decamelize($class_name);
		$cwd					= getcwd();
		$custom_controller_path	= "{$classes_dir}/{$ctrl_cls_name}.php";
		$pages_path				= "pages/{$pages_dir}/default.php";
		
		if (preg_match('/core$/', $cwd))
		{
			$cwd					= str_replace('core', '', $cwd);
			$custom_controller_path	= $cwd . $custom_controller_path;
			$pages_path				= $cwd . $pages_path;
		}

		if (file_exists($custom_controller_path))
		{
			$GLOBALS['Controller'] = new $ctrl_cls_name($this);
		}
		else
		{
			$GLOBALS['Controller'] = new ASController($this);
		}
	}

	/**
	 * Get an instance of this class.
	 * 
	 * @access public
	 * @param String The page name
	 * @return ASPageRequest
	 */
	public static function instance($page_name=null)
	{
		/*
		if (is_null(self::$instance) && is_null($page_name))
		{
			self::$instance = new ASPageRequest();
		}
		else
		{
			self::$instance = new ASPageRequest($page_name);
		}
		*/
		
		if (is_null(self::$instance))
		{
			self::$instance = new ASPageRequest($page_name);
		}

		return self::$instance;
	}

	/**
	 * Get the name of the model for this page.
	 * 
	 * Given the URL 'http://example.com/app/first_names/list' the model name 
	 * would be FirstNamesModel.
	 * 
	 * @access private
	 * @return String
	 */
	private function modelName()
	{
		return $this->model_name;
	}

	/**
	 * Notify any objects observing this one.
	 * 
	 * @access public
	 * @param Array The information being sent to the observers.
	 * @return void
	 */
    public function notifyObservables($info)
    {
		if (sizeof($this->observers))
		{
			foreach ($this->observers as $observer)
			{
				$observer->updateFromObserver($info);
			}
		}
	}

	/**
	 * The 'page' part of the URL.
	 *
	 * @access public
	 * @return String
	 */
	public function page()
	{
		return $this->URLParts(self::AS_PAGE_REQ_URL_PART_PAGE);
	}

	/**
	 * Get the page class name.
	 * 
	 * Given the URL 'http://example.com/app/first_names/list' the page class 
	 * name would be FirstNames.  This is mostly used internally to determine 
	 * the Controller and Model class names.
	 * 
	 * @access public
	 * @return String
	 */
	public function pageClassName()
	{
		return $this->page_class_name;
	}

	/**
	 * Get the data for this page.
	 * 
	 * @see setPageData()
	 * @return Array
	 */
	public function pageData()
	{
		return $this->page_data;
	}
	
	/**
	 * Get the page app name for this page.
	 * 
	 * @access public
	 * @return String
	 */
	public function pageAppName()
	{
		return preg_replace('/_$/', '', $this->pageName() . '_' . $this->appName());
	}

	/**
	 * Get the full path to the action for this page.
	 * 
	 * @access public
	 * @return String
	 */
	public function pathToAction()
	{
		return APP_HREF . '/' . $this->pageName() . '/';
	}

	/**
	 * Get the page name.
	 * 
	 * Given the URL 'http://example.com/app/first_names/list' the page name 
	 * would be first_names.
	 * 
	 * @access public
	 * @return String
	 */
	public function pageName()
	{
		return $this->page_name;
	}

	/**
	 * Parse the URL for the appropriate data segments.
	 * 
	 * A typical URL might look like 
	 * 'http://example.com/app/first_names/update/1234'.  The page_name would 
	 * be 'first_names', action would be 'update', and id would be '1234'.
	 * 
	 * @access private
	 * @return void 
	 */
	private function parseURL()
	{		
		$this->url_parts = array_merge(
			$this->url_parts
		  , array_values
		  	(
		  		array_diff
		  		(
		  			explode('/', $_SERVER['REQUEST_URI'])
		  		  , explode('/', $_SERVER['SCRIPT_NAME'])
		  		)
		  	)
		);
		
		$page		= $this->URLParts(self::AS_PAGE_REQ_URL_PART_PAGE);
		$page_dir	= "pages/{$page}";
		$page_file	= "pages/{$page}.php";
		
		if (false == is_null($page)
		&& (is_dir($page_dir)
		|| file_exists($page_file)))
		{
			ASLog::instance()->debug("Setting page name to '{$page}'");
			$this->setPageName($page);
		}
		else
		{
			ASLog::instance()->debug
			(
				"Setting page name to default ('"
			  . ASConfig::instance()->pagesDefault() . "')"
			);
			
			$this->setPageName
			(
				ASConfig::instance()->pagesDefault()
			);
		}

		if ('' == $this->URLParts(self::AS_PAGE_REQ_URL_PART_ACTION))
		{
			$this->setAction('home');
		}
		else
		{
			$this->setAction
			(
				$this->URLParts(self::AS_PAGE_REQ_URL_PART_ACTION)
			);
		}

		$this->setPageClassName($this->pageName());
		$this->setAppName($this->URLParts(self::AS_PAGE_REQ_URL_PART_PAGE));
		$this->setTemplate(ASConfig::instance()->pagesTemplate());
		$this->setId($this->URLParts(self::AS_PAGE_REQ_URL_PART_ID));
		$this->setFormActionURL(
			sprintf(
				"%s/%s%s%s"
			  , $_SERVER['SCRIPT_NAME']
			  , $this->pageName()
			  , (is_null($this->action())   ? '' : '/' . $this->action())
			  , (is_null($this->id()) ? '' : '/' . $this->id())
			)
		);
	}

	/**
	 * Remove an observer from the observers stack.
	 * 
	 * @param ASObservable The observer to remove
	 * @return void
	 */
    public function removeObserver($obs)
    {
		if (sizeof($this->observers))
		{
			foreach ($this->observers as $observer)
			{
				if ($observer === $obs)
				{
					unset($observer);
				}
			}
		}
	}

	/**
	 * Set the action for this page.
	 * 
	 * Given the URL 'http://example.com/app/first_names/list' the action would 
	 * be 'list'.
	 * 
	 * @param String The action
	 * @return void
	 */
	public function setAction($action)
	{
		$this->action = $action;
	}

	/**
	 * Set the confirmation for this page.
	 * 
	 * @param String The confirmation
	 * @return void
	 */
	public function setConfirmation($confirmation)
	{
		$this->confirmation = $confirmation;
	}

	/**
	 * Set the app name for this page. 
	 * 
	 * @param String The app name
	 * @return void
	 */
	private function setAppName($app_name)
	{
		$this->app_name = $app_name;
	}

	/**
	 * Set the error for this page.
	 * 
	 * @param String The error.
	 * @return void
	 */
	public function setError($error)
	{
		$this->error = $error;
	}

	/**
	 * Set the ID of this page.
	 * 
	 * @param Integer The ID.
	 * @return void
	 */
	public function setId($id)
	{
		$this->id = $id;
		$this->url_parts[self::AS_PAGE_REQ_URL_PART_ID] = $id;
	}

	/**
	 * Set the form action URL.
	 * 
	 * @param String The URL.
	 * @return void
	 */
	private function setFormActionURL($form_action_url)
	{
		$this->form_action_url = $form_action_url;
	}

	/**
	 * Set the model name.
	 * 
	 * Given the string 'user_account' this method will set the model name 
	 * to be 'UserAccountModel'.
	 * 
	 * @param String The base name of the model.
	 * @return Void
	 */
	public function setModelName($base_name)
	{
		$this->model_name = ucfirst($base_name) . 'Model';
	}

	/**
	 * Set the page class name.
	 * 
	 * Given the string 'user_account' this method will set the page class name
	 * to be 'UserAccount'.
	 * 
	 * @param String The page name.
	 * @return void
	 */
	public function setPageClassName($page_name)
	{
		$this->page_class_name = ASString::camelize($page_name);
	}

	/**
	 * Set the page data.
	 * 
	 * @access public
	 * @param Array The page data.
	 * @return Void
	 */
	public function setPageData($page_data)
	{
		$this->page_data = $page_data;
	}
		
	/**
	 * Set the page name.
	 * 
	 * @access public
	 * @param String The page name.
	 * @return void
	 */
	public function setPageName($page_name)
	{
		$this->page_name = $page_name;
	}

	/**
	 * Set the template for this page.
	 * 
	 * @access private
	 * @param String The template file name.
	 * @return void
	 */
	public function setTemplate($template)
	{
		$this->template = $template;
	}

	/**
	 * Get the full path and file name of the template for this page.
	 * 
	 * @access public
	 * @return String
	 */
	public function template()
	{
		return "pages/{$this->template}";
	}

	/**
	 * Get the URL data for this page.
	 * 
	 * @access public
	 * @example ../../docs/examples/ASPageRequest_URLParts.php
	 * @param Integer The index for the part of the URL to get.
	 * @return Mixed
	 */
	public function URLParts($idx=null)
	{
//		return is_null($idx)
//			? $this->url_parts 
//			: (isset($this->url_parts[$idx]) ? $this->url_parts[$idx] : null);
		$return_val = null;
		
		if (is_null($idx))
		{
			$return_val = $this->url_parts;
		}
		else if (isset($this->url_parts[$idx]))
		{
			$return_val = $this->url_parts[$idx];
		}
		
		return $return_val;
	}
}
?>
