<?php
/**
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is Alchitect Core Server.
 * 
 * The Initial Developer of the Original Code is Darrell Brogdon.
 * Portions created by Darrell Brogdon are Copyright (C) 2002-2009 Alchisoft, LLC
 * All Rights Reserved.
 * 
 * Contributor(s): .
 * 
 * @package AlchitectCoreServer
 */
/**
 * ASString
 * 
 * A static class for string manipulation.
 * 
 * @author Darrell Brogdon <darrell@brogdon.net>
 * @copyright (c)2002-2009 Alchisoft, LLC
 * @package AlchitectCoreServer
 * @version $Id: $
 */
class ASString
{
	/**
	 * A regex constant for matchging domain names.
	 * 
	 * @access public
	 * @final
	 * @todo This should be moved somewhere else.  Not sure where.
	 */
	const AS_DOMAIN_REGEX 
		= '^([a-zA-Z0-9]([a-zA-Z0-9\-]{0,61}[a-zA-Z0-9])?\.)+[a-zA-Z]{2,6}$';
	
	/**
	 * A regex for matching valid email addresses.
	 * 
	 * @example ASString_VALID_EMAIL_ADDR.php
	 * @final
	 */
	const AS_VALID_EMAIL_ADDR = "[\w0-9._%+-]+@[\w0-9.-]+\.[\w]{2,4}";
	
	
	/**
	 * A regex for matching valid RFC-8601 dates
	 * 
	 * @example ASString_VALID_RFC8601_DATE.php
	 * @final 
	 */
	const AS_VALID_RFC8601_DATE 
		= "[\d]{4}-[\d]{2}-[\d]{2} [\d]{2}:[\d]{2}:[\d]{2}[Z]*";

	/**
	 * The constructor.
	 * 
	 * Throws an exception as methods of this class should only ever be 
	 * called statically.
	 *
	 * @access public
	 * @return Void
	 */
	public function __construct()
	{
		throw new Exception('ASString is a static class.');
	}
	
	/**
	 * Camelize a string.
	 * 
	 * This method will take a string such as 'hello_world' and turn
	 * it into the camel case alternative 'HelloWorld'.
	 * 
	 * If the $perserve_ws flag is enabled then 'hello world' will be turned 
	 * into 'Hello World'.
	 *
	 * @access public
	 * @example ../../docs/examples/ASString_camelize.php
	 * @param String The raw string.
	 * @param String The separator character(s).
	 * @param Boolean Preserve any whitespace in the string.
	 * @return String
	 * @static 
	 */
	public static function camelize($str, $separator='_', $preserve_ws=false)
	{
		$new_str = str_replace($separator, ' ', $str);
		$new_str = ucwords($new_str);

		if (false == $preserve_ws)
		{
			$new_str = str_replace(' ', '', $new_str);
		}
		
		return $new_str;
	}
	
	/**
	 * De-camelize a string.
	 * 
	 * This method takes a "camelized" string such as 'HelloWorld' and
	 * turns it into the alternative 'hello_world' using the underscore (_) 
	 * character as the glue by default.
	 *
	 * @access public
	 * @example ../../docs/examples/ASString_decamelize.php
	 * @param String $str The camelized string
	 * @param String $glue The glue character(s)
	 * @return String
	 * @static 
	 */
	public static function decamelize($str, $glue='_')
	{
		$counter  = 0;
		$uc_chars = '';
		$new_str  = array();
		$str_len  = strlen($str);
		
		for ($x=0; $x<$str_len; ++$x)
		{
			$ascii_val = ord($str[$x]);
		
			if ($ascii_val >= 65 && $ascii_val <= 90)
			{
				$uc_chars .= $str[$x];
			}
		}
		
		$tok = strtok($str, $uc_chars);
		
		while ($tok !== false)
		{
			$new_char  = chr(ord($uc_chars[$counter]) + 32);
			$new_str[] = $new_char . $tok;
			$tok       = strtok($uc_chars);
			
			++$counter;
		}
		
		return implode($new_str, $glue);
	}

	/**
	 * Generate an RFC 41222 compliant UUID.
	 * 
	 * Thanks to mimec for posting this to 
	 * http://us.php.net/manual/en/function.uniqid.php#69164
	 * 
	 * @access public
	 * @return String
	 */
	public static function uuid()
	{
		return sprintf(
			'%04x%04x-%04x-%04x-%04x-%04x%04x%04x'
		  , mt_rand(0, 0xffff), mt_rand(0, 0xffff), mt_rand(0, 0xffff)
		  , mt_rand(0, 0x0fff) | 0x4000
		  , mt_rand(0, 0x3fff) | 0x8000
		  , mt_rand(0, 0xffff), mt_rand(0, 0xffff), mt_rand(0, 0xffff)
		);
	}
}
?>