/**
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is Alchitect Core Server.
 * 
 * The Initial Developer of the Original Code is Darrell Brogdon.
 * Portions created by Darrell Brogdon are Copyright (C) 2002-2009 Darrell Brogdon.
 * All Rights Reserved.
 * 
 * Contributor(s): .
 * 
 * @package AlchitectCoreUI
 */
/** 
 * @fileoverview This class is for managing authentication of an application
 * between the client and the server.  When a user logs in an authentication
 * hash is generated and sent back to the client to store in a cookie.  Every
 * time the client makes a request to the server it must provide this hash so
 * that the server can validate it.  If validation succeeds then the server 
 * will perform the requested action.
 * @author Darrell Brogdon &lt;{@link http://mailto:darrell@brogdon.net darrell@brogdon.net}&gt;
 * @copyright 2002-2009 Darrell Brogdon - All Rights Reserved
 * @version 1.0.0.0 
 */
/**
 * @class The Authentication class.
 * Initializes the private auth hash.  
 * @constructor
 * @returns An instance of ASAuthentication.
 * @type Object
 */
function ASAuthentication(username, password)
{
	if (null != username && null != password)
	{
		this.setAuthHash(username, password);
	}
}

// Extend ASObject.
ASAuthentication.prototype = new ASObject();

/**
 * Get the auth hash from the cookies.
 *
 * @return The auth hash.
 * @type String
 */
ASAuthentication.prototype.authHash = function()
{
	return $.cookie('auth_hash') || null;
}

/**
 * Get the auth ID from the cookies.
 *
 * @return The auth ID.
 * @type String
 */
ASAuthentication.prototype.authID = function()
{
	return $.cookie('auth_id') || null;
}

/**
 *
 */
ASAuthentication.prototype.logout = function()
{
	$.cookie('auth_id', null);
	$.cookie('auth_hash', null);
}

/**
 * Set the auth hash obtained from the server.
 *
 * @param The username.
 * @param The unencrypted password.
 * @return The auth hash.
 * @type String
 */
ASAuthentication.prototype.setAuthHash = function(username, password)
{
	var data = JSON.stringify({
		'form_action': {'value': 'set_authentication'}, 'username': {'value': username}, 'password': {'value': password}
	});

	$.post(
		URLPath() + '/server.php'
	  , 'form_name=authentication&json=' + data
	  , function(xhr_data, status)
	  	{
			$.cookie('auth_hash',	xhr_data.auth_hash);
			$.cookie('auth_id',		username);
		}
	  , "json"
	);

	return (null == this.authID() || null == this.authHash());
}

ASAuthentication.prototype.verifyAuthHash = function(username, auth_hash)
{
	var data = JSON.stringify({
		'form_action':	{'value': 'verify_authentication'}
	  , 'username':		{'value': (null == username  ? this.authID()   : username)}
	  , 'auth_hash':	{'value': (null == auth_hash ? this.authHash() : auth_hash)}
	});

	var _verified;

	$.ajax({
		type: "POST"
	  , async: false
	  , url: URLPath() + '/server.php'
	  , data: 'form_name=authentication&json=' + data
	  , dataType: 'json'
	  , success: function(xhr_data, status)
		{
//			console.log('VAH SUCCESS XHR DATA: %o', xhr_data.verified);
			_verified = xhr_data.verified;
		}
	  , error: function(xhr, textStatus, errorThrown)
	  	{
			console.log('VAH ERROR: %o, %s, %o', xhr, textStatus, errorThrown);
		}
	});

	return _verified;
}
