/**
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is Alchitect Core Server.
 * 
 * The Initial Developer of the Original Code is Darrell Brogdon.
 * Portions created by Darrell Brogdon are Copyright (C) 2002-2009 Darrell Brogdon.
 * All Rights Reserved.
 * 
 * Contributor(s): .
 * 
 * @package AlchitectCoreUI
 */
/** 
 * @fileoverview This class is used for sanitization of strings, usually within
 * form field text and textarea elements.
 * @author Darrell Brogdon &lt;{@link http://mailto:darrell@brogdon.net darrell@brogdon.net}&gt;
 * @copyright 2002-2009 Darrell Brogdon - All Rights Reserved
 * @version 1.0.0.0 
 */
/**
 * @class A class for sanitizing strings.
 * @constructor
 * @returns An instance of ASSanitizer.
 * @type Object
 */
function ASSanitizer()
{
}

// Extend ASObject
ASSanitizer.prototype = new ASObject();

/**
 * Encode characters with a high ASCII value.
 *  
 * @private
 * @param The string
 * @returns String
 * @type Void
 */
ASSanitizer.prototype._encodeHigh = function(str)
{
	var x = 0;
	var new_str = '';
	
	for (; x<str.length; x++)
	{
		var char_code	= str.charCodeAt(x);
		var char		= str.charAt(x);
		
		if (char_code > 127)
		{
			new_str += escape(char);
		}
		else
		{
			new_str += char;
		}
	}
	
	return new_str;
}

/**
 * Encode characters with a low ASCII value.
 * 
 * Chances are most of the characters in this range won't be found in a form
 * field but it can't hurt to encode for them anyway.
 *  
 * @private
 * @param The string
 * @returns String
 * @type Void
 */
ASSanitizer.prototype._encodeLow = function(str)
{
	var x = 0;
	var new_str = '';
	
	for (; x<str.length; x++)
	{
		var char_code	= str.charCodeAt(x);
		var char		= str.charAt(x);
		
		if (char_code >= 0 && char_code < 32)
		{
			new_str += escape(char);
		}
		else
		{
			new_str += char;
		}
	}
	
	return new_str;
}

/**
 * Strip characters with a high ASCII value.
 *  
 * @private
 * @param The string
 * @returns String
 * @type Void
 */
ASSanitizer.prototype._stripHigh = function(str)
{
	var x = 0;
	var len = str.length;
	var new_str = str;
	
	for (; x<len; x++)
	{
		var char_code	= str.charCodeAt(x);
		var char		= str.charAt(x);
		
		if (char_code > 127)
		{
			eval("new_str = new_str.replace(/" + char + "*/gi, '')");
		}
	}
	
	return new_str;
}

/**
 * Strip characters with a low ASCII value.
 * 
 * Chances are most of the characters in this range won't be found in a form
 * field but it can't hurt to strip them anyway.
 *  
 * @private
 * @param The string
 * @returns String
 * @type Void
 */
ASSanitizer.prototype._stripLow = function(str)
{
	var x = 0;
	var len = str.length;
	var new_str = str;
	
	for (; x<len; x++)
	{
		var char_code	= str.charCodeAt(x);
		var char		= str.charAt(x);
		
		if (char_code >= 0 && char_code < 32)
		{
			eval("new_str = new_str.replace(/" + char + "*/gi, '')");
		}
	}
	
	return new_str;
}

/**
 * Sanitize a string
 * 
 * Looks at the given string and strips away any characters with a high or low
 * ASCII value.  High is greater than 127, low is 0-32.
 * 
 * @public
 * @param The string
 * @returns String
 * @type Void
 */
ASSanitizer.prototype.sanitizeString = function(str_val)
{
	str_val = this._stripHigh(str_val);
	str_val = this._stripLow(str_val);
		
	return str_val;
}

/**
 * Sanitize a string
 * 
 * Looks at the given string and encodes any characters with a high or low
 * ASCII value.  High is greater than 127, low is 0-32.
 * 
 * @public
 * @param The string
 * @returns String
 * @type Void
 */
ASSanitizer.prototype.sanitizeEncoded = function(str_val)
{
	str_val = this._encodeHigh(str_val);
	str_val = this._encodeLow(str_val);
		
	return str_val;
}

/**
 * Sanitize an email string
 * 
 * Looks at the given string and strips away any characters that aren't valid
 * in an email address.
 * 
 * @public
 * @param The string
 * @returns String
 * @type Void
 */
ASSanitizer.prototype.sanitizeEmail = function(str_val)
{
	var re = /[^\w\d\!\#\$\%\&\'\+\-\/\=\?\^\_\`\{\|\\\}\~\@\.\[\]]*/g;
		
	str_val = str_val.replace(re, '');
		
	return str_val;
}

/**
 * Sanitize a URL string
 * 
 * Looks at the given string and strips away any characters that aren't valid
 * in a URL.
 * 
 * @public
 * @param The string
 * @returns String
 * @type Void
 */
ASSanitizer.prototype.sanitizeURL = function(str_val)
{
	var re = /[^\w\d\$\-\_\.\+\!\*\'\(\)\,\{\}\|\\\^\~\[\]\`\<\>\#\%\"\;\/\?\:\@\&\=\.]*/g;
	
	str_val = str_val.replace(re, '');

	return str_val;
}

/**
 * Sanitize a integer string
 * 
 * Looks at the given string and strips away any characters that aren't valid
 * in an integer.
 * 
 * @public
 * @param The string
 * @returns String
 * @type Void
 */
ASSanitizer.prototype.sanitizeInt = function(str_val)
{
	var re = /[^\d+-]*/g;
	
	str_val = str_val.replace(re, '');

	return str_val;
}

/**
 * Sanitize a float string
 * 
 * Looks at the given string and strips away any characters that aren't valid
 * in a float.
 * 
 * @public
 * @param The string
 * @returns String
 * @type Void
 */
ASSanitizer.prototype.sanitizeFloat = function(str_val)
{
	var re = /[^\d+-\.,eE]*/g;
	
	str_val = str_val.replace(re, '');

	return str_val;
}

/**
 * Sanitize a string
 * 
 * Looks at the given string and escapes any characters that might be invalid
 * when inserting or updating a database record.
 * 
 * @public
 * @param The string
 * @returns String
 * @type Void
 */
ASSanitizer.prototype.sanitizeMagicQuotes = function(str_val)
{
	return str_val.replace(/(["'\\]*)/g, function(_char){return escape(_char)});
}

/**
 * Sanitize all input text and textarea fields of a given form.
 *
 * Loops through each input field of the given form and identifies any text
 * or textarea fields.  For each one that's found it looks for a 'sanitize'
 * attribute and performs the appropriate sanitization method.
 *
 * @public
 * @param The form object
 * @returns Void.
 * @type Void
 */
ASSanitizer.prototype.sanitizeInputFields = function(frm)
{
	var self = this;
	
	frm.find(':input').each(function(idx, element)
	{
		var element = $(element);

		if ('submit' == element.attr('type') 
		 || 'reset' == element.attr('type')
		 || undefined == element.attr('sanitize')
		 || '' == element.attr('sanitize')
		)
		{
			return;
		}

		switch (element.attr('sanitize').toLowerCase())
		{
			case 'special_chars':
			case 'string':
			case 'stripped':
			{
				var new_val = self.sanitizeString(element.val());
				break;
			}
			case 'encoded':
			{
				var new_val = self.sanitizeEncoded(element.val());
				break;
			}
			case 'email':
			case 'email_address':
			{
				var new_val = self.sanitizeEmail(element.val());
				break;
			}
			case 'url':
			{
				var new_val = self.sanitizeURL(element.val());
				break;
			}
			case 'number_int':
			case 'number_integer':
			{
				var new_val = self.sanitizeInt(element.val());
				break;
			}
			case 'number_float':
			{
				var new_val = self.sanitizeFloat(element.val());
				break;
			}
			case 'magic_quotes':
			{
				var new_val = self.sanitizeMagicQuotes(element.val());
				break;
			}
			case 'raw':
			default:
			{
				break;
			}
		}
		
		element.val(new_val);
	});
}