/**
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is Alchitect Core Server.
 * 
 * The Initial Developer of the Original Code is Darrell Brogdon.
 * Portions created by Darrell Brogdon are Copyright (C) 2002-2009 Darrell Brogdon.
 * All Rights Reserved.
 * 
 * Contributor(s): .
 * 
 * @package AlchitectCoreUI
 */
/** 
 * @fileoverview All Unit Tests must extend this class.
 * @author Darrell Brogdon &lt;{@link http://mailto:darrell@brogdon.net darrell@brogdon.net}&gt;
 * @copyright 2002-2009 Darrell Brogdon - All Rights Reserved
 * @version 1.0.0.0 
 */
/**
 * @class A parent class for Unit Testing.  
 * @constructor
 * @param Whether to log the results of the individual tests
 * to the console.
 * @returns An instance of ASUnitTest.
 * @type Object
 */
function ASUnitTest(log_to_console)
{
	/**
	 * Whether to log test info to the console.
	 *
	 * @type Boolean
	 */
	this.log_to_console = log_to_console || false;

	/**
	 * An array of failed tests.
	 *
	 * @type Array
	 */
	this.failed_list	= [];

	/**
	 * A count of the number of tests that failed.
	 *
	 * @type Integer
	 */
	this.failed			= 0;

	/**
	 * A count of the number of tests that passed.
	 *
	 * @type Integer
	 */
	this.passed			= 0;
}

// Extend ASObject
//ASUnitTest.prototype = new ASObject();

/**
 * Run all the tests.
 *
 * This method looks at all the methods of the unit test subclass
 * and for each one that begins with 'test' executes that method.
 * 
 * If the method being executed returns 'true' then the 'passed' 
 * property is incremented.  Otherwise the failed property is
 * incremented and the method is added to the 'failed_list' stack.
 *
 * @returns Void.
 * @type Void
 */
ASUnitTest.prototype.run = function()
{
	if (this.setup)
	{
		this.setup();
	}

	for (method in this)
	{
		if (method.match(/^test/))
		{
			if (this.log_to_console)
			{
				console.log("TESTING: %s()", method);
			}

			if (true === this[method]())
			{
				this.passed++;
			}
			else
			{
				this.failed++;
				this.failed_list.push(method);
			}
		}
	}

	if (this.log_to_console)
	{
		console.log('PASSED: %d, FAILED: %d', this.passed, this.failed);
	}
}

/**
 * Assert that a given value is true.
 *
 * @returns Fail if x is false
 * @type Boolean
 */
ASUnitTest.prototype.assertTrue = function(x)	
{
	return true == x;
}

/**
 * Assert that a given value is false.
 *
 * @returns Fail if x is true
 * @type Boolean
 */
ASUnitTest.prototype.assertFalse = function(x)	
{
	return false == x;
}

/**
 * Assert that a given value is null.
 *
 * @returns Fail if x is set
 * @type Boolean
 */
ASUnitTest.prototype.assertNull = function(x)	
{
	return null == x;
}

/**
 * Assert that a given value is not null.
 *
 * @returns Fail if x not set
 * @type Boolean
 */
ASUnitTest.prototype.assertNotNull = function(x)	
{
	return null != x;
}

/**
 * Assert that a given value is an instance of the second value.
 *
 * @returns Fail if x is not the class or type t
 * @type Boolean
 */
ASUnitTest.prototype.assertIsA = function(x, t)	
{
	return x.prototype.isPrototypeOf(t);
}

/**
 * Assert that a given value is equal to the second value.
 *
 * @returns Fail if x == y is false
 * @type Boolean
 */
ASUnitTest.prototype.assertEqual = function(x, y)	
{
	return x == y;
}

/**
 * Assert that a given value is not equal to the second value.
 *
 * @returns Fail if x == y is true
 * @type Boolean
 */
ASUnitTest.prototype.assertNotEqual = function(x, y)	
{
	return x != y;
}

/**
 * Assert that a given value is identical (in value and type) 
 * to the second value.
 *
 * @returns Fail if x === y is false
 * @type Boolean
 */
ASUnitTest.prototype.assertIdentical = function(x, y)	
{
	return x === y;
}

/**
 * Assert that a given value is not identical (in value and
 * type) to the second value.
 *
 * @returns Fail if x === y is true
 * @type Boolean
 */
ASUnitTest.prototype.assertNotIdentical = function(x, y)	
{
	return x !== y;
}

/**
 * Assert that a given value matches a given regex pattern.
 *
 * @returns Fail unless the regex p matches x
 * @type Boolean
 */
ASUnitTest.prototype.assertWantedPattern = function(p, x)	
{
	return null != x.match(p);
}

/**
 * Assert that a given value does not match a given regex
 * pattern.
 *
 * @returns Fail if the regex p matches x
 * @type Boolean
 */
ASUnitTest.prototype.assertNoUnwantedPattern = function(p, x)	
{
	return null == x.match(p);
}
