/**
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is Alchitect Core Server.
 * 
 * The Initial Developer of the Original Code is Darrell Brogdon.
 * Portions created by Darrell Brogdon are Copyright (C) 2002-2009 Darrell Brogdon.
 * All Rights Reserved.
 * 
 * Contributor(s): .
 * 
 * @package AlchitectCoreUI
 */
/** 
 * @fileoverview This class is used for validation of strings, usually within
 * form field text and textarea elements.
 * @author Darrell Brogdon &lt;{@link http://mailto:darrell@brogdon.net darrell@brogdon.net}&gt;
 * @copyright 2002-2009 Darrell Brogdon - All Rights Reserved
 * @version 1.0.0.0 
 */
/**
 * @class A class for validating strings.
 * @constructor
 * @returns An instance of ASValidater.
 * @type Object
 */
function ASValidater()
{
}

// Extend ASObject
ASValidater.prototype = new ASObject();

/**
 * Validate email addresses
 * 
 * Looks at the given string and determines whether it's a valid email
 * address.
 * 
 * If the string isn't an email address then an exception is thrown with the id
 * of the element containing the string.
 * 
 * @public
 * @param The string
 * @param The id of the element containing the string.
 * @returns Void
 * @type Void
 */
ASValidater.prototype.validateEmail = function(str_val, id)
{
	var match = null != str_val.match
						(
							/^[\w0-9._%+-]+@[\w0-9.-]+\.[\w]{2,4}$/g
						);

	if (false == match)
	{
		throw {code:1000, message:"Invalid Email Address", id:id};
	}
}

/**
 * Validate a URL
 * 
 * Looks at the given string and determines whether it's a valid URL.
 * 
 * If the string isn't a URL then an exception is thrown with the id
 * of the element containing the string.
 * 
 * Thanks to bb at http://www.osix.net/modules/article/?id=586
 * for the regular expression.
 * 
 * @public
 * @param The string
 * @param The id of the element containing the string.
 * @returns Void
 * @type Void
 */
ASValidater.prototype.validateURL = function(str_val, id)
{
	var re_str = "^((https?|ftp|mailto)://)"
				//user@
			   + "?(([0-9a-z_!~*'().&=+$%-]+: )?[0-9a-z_!~*'().&=+$%-]+@)?"
			   // IP- 199.194.52.184
			   + "(([0-9]{1,3}\.){3}[0-9]{1,3}"
			   // allows either IP or domain
			   + "|"
			   // tertiary domain(s)- www.
			   + "([0-9a-z_!~*'()-]+\.)*"
			   // second level domain
			   + "([0-9a-z][0-9a-z-]{0,61})?[0-9a-z]\."
			   // first level domain- .com or .museum
			   + "[a-z]{2,6})"
			   // port number- :80
			   + "(:[0-9]{1,4})?"
			   // a slash isn't required if there is no file name
			   + "((/?)|"
			   + "(/[0-9a-z_!~*'().;?:@&=+$,%#-]+)+/?)$";
	var match = new RegExp(re_str).test(str_val);
	
	if (false == match)
	{
		throw {code:1100, message:"Invalid URL", id:id};
	}
}

/**
 * Validate integer numbers
 * 
 * Looks at the given string and determines whether it's a valid integer.
 * 
 * If the string isn't an integer then an exception is thrown with the id
 * of the element containing the string.
 * 
 * @public
 * @param The string
 * @param The id of the element containing the string.
 * @returns Void
 * @type Void
 */
ASValidater.prototype.validateInt = function(str_val, id)
{
	var return_val_one = false == isNaN(str_val);
	var return_val_two = false == isNaN(parseInt(str_val));
	
	var match = return_val_one && return_val_two;
	
	if (false == match)
	{
		throw {code:1200, message:"Invalid Integer", id:id};
	}
}

/**
 * Validate float numbers
 * 
 * Looks at the given string and determines whether it's a valid float.
 * 
 * If the string isn't a float then an exception is thrown with the id
 * of the element containing the string.
 * 
 * @public
 * @param The string
 * @param The id of the element containing the string.
 * @returns Void
 * @type Void
 */
ASValidater.prototype.validateFloat = function(str_val, id)
{
	var return_val_one = false == isNaN(str_val);
	var return_val_two = false == isNaN(parseFloat(str_val));
	
	var match = return_val_one && return_val_two;
	
	if (false == match)
	{
		throw {code:1300, message:"Invalid Float", id:id};
	}
}

/**
 * Validate IP Addresses
 * 
 * Looks at the given string and determines whether it's a valid IPv4 IP
 * address.  Currently, IPv6 is not supported.
 * 
 * If the string isn't an IP address then an exception is thrown with the id
 * of the element containing the string.
 * 
 * @public
 * @param The string
 * @param The id of the element containing the string.
 * @returns Void
 * @type Void
 */
ASValidater.prototype.validateIP = function(str_val, id)
{	
	var re_str = "^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\."
			   + "(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\."
			   + "(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\."
			   + "(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$";
	var match = new RegExp(re_str).test(str_val);
	
	if (false == match)
	{
		throw {code:1400, message:"Invalid IP Address", id:id};
	}
}

/**
 * Validate all input text and textarea fields of a given form.
 *
 * Loops through each input field of the given form and identifies any text
 * or textarea fields.  For each one that's found it looks for a 'validate'
 * attribute and performs the appropriate validation method.
 *
 * @public
 * @param The form object
 * @returns Void.
 * @type Void
 */
ASValidater.prototype.validateInputFields = function(frm)
{
	var self = this;
	
	frm.find(':input').each(function(idx, element)
	{
		var element = $(element);

		if ('submit' == element.attr('type') 
		 || 'reset' == element.attr('type')
		 || undefined == element.attr('validate')
		 || '' == element.attr('validate')
		)
		{
			return;
		}
		
		switch (element.attr('validate').toLowerCase())
		{
			case 'email':
			case 'email_address':
			{
				self.validateEmail(element.val(), element.attr('id'));
				break;
			}
			case 'url':
			{
				self.validateURL(element.val(), element.attr('id'));
				break;
			}
			case 'int':
			case 'integer':
			{
				self.validateInt(element.val(), element.attr('id'));
				break;
			}
			case 'float':
			{
				self.validateFloat(element.val(), element.attr('id'));
				break;
			}
			case 'ip':
			case 'ip_address':
			{
				self.validateIP(element.val(), element.attr('id'));
				break;
			}
			default:
			{
				break;
			}
		}
	});
}